package com.codigoparallevar.minicards.bridge.blocks;

import android.util.Log;

import com.programaker.bridge.ProgramakerBridgeConfigurationBlock;

import org.jetbrains.annotations.NotNull;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.List;

class TriggerBlockDefinition implements ProgramakerBridgeConfigurationBlock {
    private final static String LogTag = "PM TriggerBlockDef";

    private final String id;
    private final String message;
    private final List<BlockArgumentDefinition> args;
    private final String key;

    public TriggerBlockDefinition(String id, String message, List<BlockArgumentDefinition> args,
                                    String key) {
        this.id = id;
        this.message = message;
        this.args = args;
        this.key = key;
    }

    @NotNull
    @Override
    public JSONObject serialize() {
        JSONObject obj = new JSONObject();

        try {
            JSONArray arguments = new JSONArray();
            for (BlockArgumentDefinition arg : this.args) {
                arguments.put(arg.serialize());
            }

            obj.put("id", this.id);
            obj.put("function_name", this.id);
            obj.put("message", this.message);
            obj.put("arguments", arguments);
            obj.put("save_to",  JSONObject.NULL);
            obj.put("expected_value",  JSONObject.NULL);
            obj.put("block_type", "trigger");
            obj.put("key", this.key);
            obj.put("subkey", JSONObject.NULL);
        } catch (JSONException ex) {
            Log.e(LogTag, "Error serializing block definition: " + ex, ex);
        }

        return obj;
    }

    @NotNull
    @Override
    public String getFunctionName() {
        return this.id;
    }

    @Override
    public Object call(@NotNull List<?> arguments) throws Exception {
        throw new IllegalArgumentException("Non executable block");
    }
}
