package com.codigoparallevar.minicards.bridge.helpers;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.Network;
import android.net.NetworkCapabilities;
import android.net.NetworkRequest;
import android.net.wifi.SupplicantState;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;

import com.programaker.bridge.ProgramakerBridge;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

public class StatusStreamer {
    private final Context ctx;
    private final ProgramakerBridge bridge;
    private WifiNetworkCallbackReceiver networkCallback = null;

    public StatusStreamer(Context ctx, ProgramakerBridge bridge) {
        this.ctx = ctx;
        this.bridge = bridge;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            this.networkCallback = new WifiNetworkCallbackReceiver(this);
        }
    }

    public void run() {

        if (android.os.Build.VERSION.SDK_INT < android.os.Build.VERSION_CODES.LOLLIPOP) {
            return;
        }

        NetworkRequest.Builder builder = new NetworkRequest.Builder();
        builder.addTransportType(NetworkCapabilities.TRANSPORT_WIFI);

        ConnectivityManager connManager = (ConnectivityManager) this.ctx.getSystemService(Context.CONNECTIVITY_SERVICE);
        assert connManager != null;

        connManager.registerNetworkCallback(builder.build(), this.networkCallback);
    }

    public void stop() {
        ConnectivityManager connManager = (ConnectivityManager) this.ctx.getSystemService(Context.CONNECTIVITY_SERVICE);
        assert connManager != null;

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            connManager.unregisterNetworkCallback(this.networkCallback);
        }
    }

    public void onWifiNetworkConnected(Network network) {
        WifiManager wifiManager = (WifiManager) this.ctx.getSystemService(Context.WIFI_SERVICE);
        WifiInfo wifiInfo;

        wifiInfo = wifiManager.getConnectionInfo();
        String ssid = "";
        if (wifiInfo.getSupplicantState() == SupplicantState.COMPLETED) {
            // This might not return the SSID in case the user has not granted location permissions to the app
            // see: https://stackoverflow.com/a/54446042
            ssid = wifiInfo.getSSID();
        }

        Map<String, String> data = new HashMap<>();
        data.put("ssid", ssid);

        this.bridge.sendSignal("on_wifi_connected", new JSONObject(data));
    }

    public void onWifiNetworkDisconnected() {
        this.bridge.sendSignal("on_wifi_disconnected", new JSONObject());
    }
}
